@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 ###########################################################################
-#    Copyright (C) 2003, 2004 by Wido Depping
+#    Copyright (C) 2003-2005 by Wido Depping
 #    <widod@users.sourceforge.net>
 #
 # Copyright: See COPYING file that comes with this distribution
@@ -99,24 +99,13 @@ def doInstall():
     print "Copy program files...\n"
     
     try:
-        a = Popen3("cp -R bin " + prefixDir)
-        while a.poll() == -1:
-            pass
-        if a.poll() > 0:
-            raise "CopyError", "Error!!! Could not copy File. Maybe wrong permissions?"
-
-        a = Popen3("cp -R lib " + prefixDir)
-        while a.poll() == -1:
-            pass
-        if a.poll() > 0:
-            raise "CopyError", "Error!!! Could not copy File. Maybe wrong permissions?"
-
-        a = Popen3("cp -R share " + prefixDir)
-        while a.poll() == -1:
-            pass
-        if a.poll() > 0:
-            raise "CopyError", "Error!!! Could not copy File. Maybe wrong permissions?"
-
+        for tmpDir in ["bin", "lib", "share"]:
+            a = Popen3("cp -R " + tmpDir + " " + prefixDir)
+            while a.poll() == -1:
+                pass
+            if a.poll() > 0:
+                raise "CopyError", "Error!!! Could not copy File. Maybe wrong permissions?"
+                
         print "Finished copying program files.\n"
         print "LUMA installed succesfully! :)"
         
@@ -209,7 +198,7 @@ def evalArguments():
 ###############################################################################
 
 
-print "Luma 2.0 (C) 2003-2005 Wido Depping\n"
+print "Luma 2.2 (C) 2003-2005 Wido Depping\n"
 
 doImportCheck()
 print ""
