@@ -140,6 +140,7 @@ class AdvancedObjectWidget(QWidget):
 ###############################################################################
 
     def displayValues(self):
+        environment.setBusy(True)
         self.objectWidget.setText("")
         
         # Something went wrong. We have no data object.
@@ -180,6 +181,7 @@ class AdvancedObjectWidget(QWidget):
         self.objectWidget.setText(self.currentDocument)
         
         self.enableToolButtons(True)
+        environment.setBusy(False)
         
 ###############################################################################
 
@@ -217,6 +219,7 @@ class AdvancedObjectWidget(QWidget):
         tmpList.append("""<table border="0" cellpadding="1" cellspacing="1" width="100%">""")
         
         for x in attributeList:
+            environment.updateUI()
             attributeIsBinary = self.ldapDataObject.isAttributeBinary(x)
             attributeIsImage = self.ldapDataObject.isAttributeImage(x)
             attributeIsPassword = self.ldapDataObject.isAttributePassword(x)
@@ -232,6 +235,13 @@ class AdvancedObjectWidget(QWidget):
                     
             valueList = self.ldapDataObject.getAttributeValueList(x)
             
+            if None == valueList:
+                continue
+            
+            if not (len(valueList) > 0):
+                continue
+                
+            
             allowDelete = False
             if attributeIsMust:
                 if len(valueList) > 1:
@@ -246,7 +256,7 @@ class AdvancedObjectWidget(QWidget):
             if self.ldapDataObject.isAttributeMust(x, self.ldapDataObject.getObjectClasses()):
                 attributeString = "<b>" + attributeString + "</b>"
             
-            if None == valueList[0]:
+            if valueList[0] == None:
                 attributeString = """<font color="red">""" + attributeString + """</font>"""
                 
             tmpList.append("""<td bgcolor="#E5E5E5" width="35%">""" + attributeString + """</td>""")
@@ -255,10 +265,10 @@ class AdvancedObjectWidget(QWidget):
             univAttributeName = x + "__" + unicode(attributeIndex)
 
             attributeModify = True
-            if not (None == valueList[0]):
+            if not (valueList[0] == None):
                 attributeModify = not self.ldapDataObject.isAttributeValueRDN(x, valueList[0])
             
-            if None == valueList[0]:
+            if (valueList[0] == None):
                 tmpList.append("""<td bgcolor="#E5E5E5" width="60%"><font color="#ff0000">""" + 
                     unicode(self.trUtf8("Value not set.")) + """</font></td>""")
                     
@@ -275,16 +285,17 @@ class AdvancedObjectWidget(QWidget):
             
             
             for y in valueList[1:]:
+                environment.updateUI()
                 attributeIndex += 1
                 univAttributeName = x + "__" + unicode(attributeIndex)
                 
                 attributeModify = True
-                if not (None == y):
+                if not (y == None):
                     attributeModify = not self.ldapDataObject.isAttributeValueRDN(x, y)
                 
                 tmpList.append("""<tr><td width="35%"></td>""")
                 
-                if None == y:
+                if y == None:
                     tmpList.append("""<td bgcolor="#E5E5E5" width="55%"><font color="#ff0000">""" +
                         unicode(self.trUtf8("Value not set.")) + """</font></td>""")
                         
@@ -369,11 +380,11 @@ class AdvancedObjectWidget(QWidget):
         index = int(tmpList[1])
         operation = tmpList[2]
         
-        if "edit" == operation:
+        if operation == "edit":
             self.editAttribute(attributeName, index)
-        elif "delete" == operation:
+        elif operation == "delete":
             self.deleteAttribute(attributeName, index)
-        elif "export" == operation:
+        elif operation == "export":
             self.exportAttribute(attributeName, index)
 
 ###############################################################################
@@ -414,7 +425,7 @@ class AdvancedObjectWidget(QWidget):
     def editAttribute(self, attributeName, index):
         oldDN = self.ldapDataObject.getDN()
         
-        if 'RDN' == attributeName:
+        if attributeName == 'RDN':     
             self.ldapDataObject.setDN(self.baseDN)
             
         dialog = EditorFactory.getEditorWidget(self, self.ldapDataObject, attributeName, index)
@@ -423,12 +434,12 @@ class AdvancedObjectWidget(QWidget):
         
         if dialog.result() == QDialog.Accepted:
             newValue = dialog.getValue()
-            if not (None == newValue):
+            if not (newValue == None):
                 self.ldapDataObject.setAttributeValue(attributeName, index, newValue)
                 self.EDITED = True
                 self.displayValues()
         else:
-            if 'RDN' == attributeName:
+            if attributeName == 'RDN':
                 self.ldapDataObject.setDN(oldDN)
 
 ###############################################################################
@@ -462,7 +473,20 @@ class AdvancedObjectWidget(QWidget):
 ###############################################################################
 
     def aboutToChange(self):
-        pass
+        if not self.EDITED:
+            return
+            
+        result = QMessageBox.warning(None,
+            self.trUtf8("Save entry"),
+            self.trUtf8("""Do you want to save the entry?"""),
+            self.trUtf8("&OK"),
+            self.trUtf8("&Cancel"),
+            None,
+            0, -1)
+            
+        if result == 0:
+            self.saveView()
+
         
 ###############################################################################
 
@@ -605,7 +629,10 @@ class AdvancedObjectWidget(QWidget):
         
         attribute = str(dialog.attributeBox.currentText())
         showAll = dialog.enableAllBox.isChecked()
-        attributeList = Set([attribute])
+        if dialog.binaryBox.isOn():
+            attributeList = Set([attribute + ";binary"])
+        else:
+            attributeList = Set([attribute])
         
         if showAll and not(attribute in dialog.possibleAttributes):
             objectClass = str(dialog.classBox.currentText())
