@@ -26,6 +26,7 @@ from base.utils import escapeSpecialChars
 from base.utils.gui.LumaErrorDialog import LumaErrorDialog
 from base.utils.gui.DeleteDialog import DeleteDialog
 from base.utils.gui.ExportDialog import ExportDialog
+from base.utils.backend.LogObject import LogObject
 
 class BrowserWidget(QListView):
     """ Widget for browsing ldap trees. 
@@ -35,11 +36,15 @@ class BrowserWidget(QListView):
 
     def __init__(self,parent = None,name = None,fl = 0):
         QListView.__init__(self,parent,name,fl)
+        self.setSelectionMode(QListView.Extended)
 
-        self.connect(self, SIGNAL("clicked(QListViewItem*)"), self.itemClicked)
+        #self.connect(self, SIGNAL("clicked(QListViewItem*)"), self.itemClicked)
+        self.connect(self, SIGNAL("mouseButtonPressed(int, QListViewItem*, const QPoint&, int)"), self.itemClicked)
+        #self.connect(self, SIGNAL("currentChanged ( QListViewItem * )"), self.itemClicked)
         self.connect(self, SIGNAL("collapsed(QListViewItem*)"), self.itemCollapsed)
         self.connect(self, SIGNAL("expanded(QListViewItem*)"), self.itemExpanded)
-        #self.connect(self, SIGNAL("doubleClicked(QListViewItem*)"), self.itemExpanded)
+        self.connect(self, SIGNAL("rightButtonPressed(QListViewItem*, const QPoint&, int)"), self.showPopup)
+
 
         self.setRootIsDecorated(True)
         self.addColumn(self.trUtf8("Entries"))
@@ -78,9 +83,6 @@ class BrowserWidget(QListView):
         self.displayServerIcons()
 
         self.addItemWidgets = []
-
-        self.connect(self, SIGNAL("rightButtonPressed(QListViewItem*, const QPoint&, int)"), self.showPopup)
-        
         self.widgetList = []
         
         # Item for which a popup menu was openend
@@ -98,9 +100,14 @@ class BrowserWidget(QListView):
         for server in self.aliasDict.keys():
             serverMeta = self.serverListObject.getServerObject(server)
             tmpItem = self.serverDict[server]
-            if serverMeta.tls and self.aliasDict[server]:
+            
+            encryption = False
+            if not (serverMeta.encryptionMethod == u"None"):
+                encryption = True
+                
+            if encryption and self.aliasDict[server]:
                 tmpItem.setPixmap(0, self.secureAliasIcon)
-            elif serverMeta.tls:
+            elif encryption:
                 tmpItem.setPixmap(0, self.secureIcon)
             elif self.aliasDict[server]:
                 tmpItem.setPixmap(0, self.aliasIcon)
@@ -109,21 +116,22 @@ class BrowserWidget(QListView):
 
 ###############################################################################
 
-    def itemClicked(self, item):
+    def itemClicked(self, buttonNumber, item):
         """ Emit the ldap object and the server if a valid object has been
         clicked.
         """
         
+        if not (buttonNumber ==1):
+            return
+            
         if not (item == None):
             fullPath = self.getFullPath(item)
 
             success, resultList, exceptionObject = self.getLdapItem(fullPath)
-            
-            if not (None == success):
+            if not (success == None):
                 if success:
                     if len(resultList) > 0:
                         result = resultList[0]
-                        #print result.dn, result.data
                         result.serverMeta.currentBase = self.currentBase
                         self.emit(PYSIGNAL("about_to_change"), ())
                         self.emit(PYSIGNAL("ldap_result"), (deepcopy(result),))
@@ -196,7 +204,7 @@ class BrowserWidget(QListView):
             else:
                 tmpList = serverMeta.baseDN
                 
-            if None == tmpList:
+            if tmpList == None:
                 tmpList = []
                 
             for base in tmpList:
@@ -209,10 +217,12 @@ class BrowserWidget(QListView):
         """ Delete all children if a ldap object collapses.
         """
         
-        fullPath = self.getFullPath(item)
-        serverName, ldapObject = self.splitPath(fullPath)
+        #fullPath = self.getFullPath(item)
+        #serverName, ldapObject = self.splitPath(fullPath)
+        
         while item.childCount() > 0:
             item.takeItem(item.firstChild())
+        
 
 ###############################################################################
 
@@ -232,9 +242,9 @@ class BrowserWidget(QListView):
                 item = item.parent()
                 
                 # Try to get the currently used baseDN
-                if not (None == item):
+                if not (item == None):
                     tmpItem = item.parent()
-                    if None == tmpItem:
+                    if tmpItem == None:
                         self.currentBase = unicode(oldItem.text(0)).encode('utf-8')
                     
                 pathItem = unicode(item.text(0)).encode('utf-8')
@@ -243,8 +253,9 @@ class BrowserWidget(QListView):
 
             return ",".join(tmpList)
         except AttributeError, e:
-            print "Attribute Error in function 'BrowserWidget.getFullPath()'. Reason:"
-            print e
+            tmpString = "Attribute Error in function 'BrowserWidget.getFullPath()'. Reason:\n"
+            tmpString += str(e)
+            environment.logMessage(LogObject("Error", tmpString))
 
 ###############################################################################
 
@@ -345,19 +356,53 @@ class BrowserWidget(QListView):
         """ Export the selected item to ldif.
         """
         
-        fullPath = self.getFullPath(self.selectedItem())
-        success, resultList, exceptionObject = self.getLdapItem(fullPath)
-        
-        if success and (len(resultList) > 0):
-                exportDialog = ExportDialog()
-                exportDialog.initData(resultList)
-                exportDialog.exec_loop()
-        else:
+        selectedItems = []
+        listIterator = QListViewItemIterator(self)
+        while listIterator.current():
+            item = listIterator.current()
+            
+            if item.isSelected():
+                selectedItems.append(item)
+                
+            listIterator += 1
+            
+        if len(selectedItems) == 0:
+            return
+            
+        entryList = []
+        partialResults = False
+        for x in selectedItems:
+            fullPath = self.getFullPath(x)
+            
+            # check if we selected a server name
+            serverName, ldapObject = self.splitPath(fullPath)
+            if len(ldapObject) == 0:
+                continue
+                
+            success, resultList, exceptionObject = self.getLdapItem(fullPath)
+            
+            if success:
+                entryList.extend(resultList)
+            else:
+                errorMsg = self.trUtf8("Could not retrieve entry with DN %1 on server %2 for exporting.<br><br>Reason: ")
+                errorMsg = errorMsg.text().arg(ldapObject).arg(serverName)
+                errorMsg.append(str(exceptionObject))
+                environment.logMessage(LogObject("Error", errorMsg))
+                partialResults = True
+               
+        if partialResults:
             dialog = LumaErrorDialog()
-            errorMsg = self.trUtf8("Could not export item.<br><br>Reason: ")
-            errorMsg.append(str(exceptionObject))
+            errorMsg = self.trUtf8("Could not retrieve all entries for exporting. More information in the logger.")
             dialog.setErrorMessage(errorMsg)
             dialog.exec_loop()
+            return
+            
+        if len(entryList) == 0:
+            return
+            
+        exportDialog = ExportDialog()
+        exportDialog.initData(entryList)
+        exportDialog.exec_loop()
 
 ###############################################################################
 
@@ -365,66 +410,152 @@ class BrowserWidget(QListView):
         """ Export the whole subtree to ldif.
         """
         
-        fullPath = self.getFullPath(self.selectedItem())
-        success, resultList, exceptionObject = self.getLdapItemChildren(fullPath, 1)
-        
-        if success and (len(resultList) > 0):
-            exportDialog = ExportDialog()
-            exportDialog.initData(resultList)
-            exportDialog.exec_loop()
-        else:
+        selectedItems = []
+        listIterator = QListViewItemIterator(self)
+        while listIterator.current():
+            item = listIterator.current()
+            
+            if item.isSelected():
+                selectedItems.append(item)
+                
+            listIterator += 1
+            
+        if len(selectedItems) == 0:
+            return
+            
+        entryList = []
+        partialResults = False
+        for x in selectedItems:
+            fullPath = self.getFullPath(x)
+            
+            # check if we selected a server name
+            serverName, ldapObject = self.splitPath(fullPath)
+            if len(ldapObject) == 0:
+                continue
+                
+            success, resultList, exceptionObject = self.getLdapItemChildren(fullPath, 1)
+            
+            if success:
+                entryList.extend(resultList)
+            else:
+                errorMsg = self.trUtf8("Could not retrieve entry with DN %1 on server %2 for exporting.<br><br>Reason: ")
+                errorMsg = errorMsg.text().arg(ldapObject).arg(serverName)
+                errorMsg.append(str(exceptionObject))
+                environment.logMessage(LogObject("Error", errorMsg))
+                partialResults = True
+               
+        if partialResults:
             dialog = LumaErrorDialog()
-            errorMsg = self.trUtf8("Could not export items.<br><br>Reason: ")
-            errorMsg.append(str(exceptionObject))
+            errorMsg = self.trUtf8("Could not retrieve all entries for exporting. More information in the logger.")
             dialog.setErrorMessage(errorMsg)
             dialog.exec_loop()
-        
+            return
+            
+        if len(entryList) == 0:
+            return
+            
+        exportDialog = ExportDialog()
+        exportDialog.initData(entryList)
+        exportDialog.exec_loop()
 
 ###############################################################################
 
     def exportItemAll(self):
         """ Export the whole subtree to ldif, together with all its parents.
+        
+        TODO: The code produces multiple exports of some entries if the 
+        selections are on different levels in the same subtree.
         """
         
-        
-        itemList = []
-        stringList = []
-        searchError = False
-        
-        currentItem = self.selectedItem()
-        fullPath = self.getFullPath(currentItem)
-        exceptionObject = None
-        
-        parentDNList = self.getParents(currentItem)
-        for x in parentDNList:
-            success, resultList, exceptionObject = self.getLdapItem(x)
-            if success:
-                itemList.extend(resultList)
-            else:
-                searchError = True
-                break
+        selectedItems = []
+        listIterator = QListViewItemIterator(self)
+        while listIterator.current():
+            item = listIterator.current()
+            
+            if item.isSelected():
+                selectedItems.append(item)
                 
-        if not searchError:
-            success, subtreeList, exceptionObject = self.getLdapItemChildren(fullPath, 1)
+            listIterator += 1
+            
+        if len(selectedItems) == 0:
+            return
+            
+        # remove duplicated items
+        tmpDict = {}
+        for x in selectedItems:
+            tmpDict[x] = None
+        selectedItems = tmpDict.keys()
+        
+        
+            
+        parentDNList = []
+        for x in selectedItems:
+            parentDNList.extend(self.getParents(x))
+            
+        # remove duplicated items for parents
+        tmpDict = {}
+        for x in parentDNList:
+            tmpDict[x] = None
+        parentDNList = tmpDict.keys()
+        
+        # get ldap entries for the parentsa of the entries to be exported
+        parentEntries = []
+        partialResults = False
+        for x in parentDNList:
+            fullPath = self.getFullPath(x)
+            
+            # check if we selected a server name
+            serverName, ldapObject = self.splitPath(fullPath)
+            if len(ldapObject) == 0:
+                continue
+                
+            success, resultList, exceptionObject = self.getLdapItem(fullPath)
             
             if success:
-                itemList.extend(subtreeList)
-                if len(itemList) > 0:
-                    exportDialog = ExportDialog()
-                    exportDialog.initData(itemList)
-                    exportDialog.exec_loop()
+                parentEntries.extend(resultList)
             else:
-                dialog = LumaErrorDialog()
-                errorMsg = self.trUtf8("Could not export items.<br><br>Reason: ")
+                errorMsg = self.trUtf8("Could not retrieve entry with DN %1 on server %2 for exporting.<br><br>Reason: ")
+                errorMsg = errorMsg.text().arg(ldapObject).arg(serverName)
                 errorMsg.append(str(exceptionObject))
-                dialog.setErrorMessage(errorMsg)
-                dialog.exec_loop()
-        else:
+                environment.logMessage(LogObject("Error", errorMsg))
+                partialResults = True
+        
+        # get ldap entries for the selected items
+        entryList = []
+        for x in selectedItems:
+            fullPath = self.getFullPath(x)
+            
+            # check if we selected a server name
+            serverName, ldapObject = self.splitPath(fullPath)
+            if len(ldapObject) == 0:
+                continue
+                
+            success, resultList, exceptionObject = self.getLdapItemChildren(fullPath, 1)
+            
+            if success:
+                entryList.extend(resultList)
+            else:
+                errorMsg = self.trUtf8("Could not retrieve entry with DN %1 on server %2 for exporting.<br><br>Reason: ")
+                errorMsg = errorMsg.text().arg(ldapObject).arg(serverName)
+                errorMsg.append(str(exceptionObject))
+                environment.logMessage(LogObject("Error", errorMsg))
+                partialResults = True
+               
+        if partialResults:
             dialog = LumaErrorDialog()
-            errorMsg = self.trUtf8("Could not export items.<br><br>Reason: ")
-            errorMsg.append(str(exceptionObject))
+            errorMsg = self.trUtf8("Could not retrieve all entries for exporting. More information in the logger.")
             dialog.setErrorMessage(errorMsg)
             dialog.exec_loop()
+            return
+            
+        if len(entryList) == 0:
+            return
+           
+        entryList.extend(parentEntries)
+        entryList.sort()
+        exportDialog = ExportDialog()
+        exportDialog.initData(entryList)
+        exportDialog.exec_loop()
 
 ###############################################################################
 
@@ -432,31 +563,54 @@ class BrowserWidget(QListView):
         """ Delete selected item from the server.
         """
         
-        item = self.selectedItem()
-        parent = item.parent()
-        fullPath = self.getFullPath(item)
-        serverName, ldapObject = self.splitPath(fullPath)
-        if len(ldapObject) == 0:
-            return None
+        selectedItems = []
+        listIterator = QListViewItemIterator(self)
+        while listIterator.current():
+            item = listIterator.current()
             
-        success, resultList, exceptionObject = self.getLdapItem(fullPath)
-        
-        if success:
-            if len(resultList) > 0:
-                deleteDialog = DeleteDialog()
-                deleteDialog.initData(resultList)
-                deleteDialog.exec_loop()
-                parent.setOpen(0)
-                parent.setOpen(1)
-                self.setSelected(parent, True)
-                self.itemClicked(parent)
-        else:
+            if item.isSelected():
+                selectedItems.append(item)
+                
+            listIterator += 1
+            
+        if len(selectedItems) == 0:
+            return
+            
+        entryList = []
+        partialResults = False
+        for x in selectedItems:
+            fullPath = self.getFullPath(x)
+            
+            # check if we selected a server name
+            serverName, ldapObject = self.splitPath(fullPath)
+            if len(ldapObject) == 0:
+                continue
+                
+            success, resultList, exceptionObject = self.getLdapItem(fullPath)
+            
+            if success:
+                entryList.extend(resultList)
+            else:
+                errorMsg = self.trUtf8("Could not retrieve entry with DN %1 on server %2 for deletion.<br><br>Reason: ")
+                errorMsg = errorMsg.text().arg(ldapObject).arg(serverName)
+                errorMsg.append(str(exceptionObject))
+                environment.logMessage(LogObject("Error", errorMsg))
+                partialResults = True
+               
+        if partialResults:
             dialog = LumaErrorDialog()
-            errorMsg = self.trUtf8("Could not retrieve entry for deletion.<br><br>Reason: ")
-            errorMsg.append(str(exceptionObject))
+            errorMsg = self.trUtf8("Could not retrieve all entries for deletion. More information in the logger.")
             dialog.setErrorMessage(errorMsg)
             dialog.exec_loop()
-
+            return
+            
+        if len(entryList) == 0:
+            return
+            
+        
+        deleteDialog = DeleteDialog()
+        deleteDialog.initData(entryList)
+        deleteDialog.exec_loop()
 
 ###############################################################################
 
@@ -464,6 +618,8 @@ class BrowserWidget(QListView):
         """ Display popup menu.
         """
         
+        #self.itemClicked(tmpItem)
+        #self.emit(SIGNAL("clicked(QListViewItem*)"), (tmpItem,))
         self.popupItem = tmpItem
         
         tmpDirObject = environment.lumaInstallationPrefix
@@ -474,6 +630,22 @@ class BrowserWidget(QListView):
         server, dn = self.splitPath(itemPath)
         
         if not (tmpItem == None):
+            # try to find how many items are selected
+            multipleSelected = False
+            listIterator = QListViewItemIterator(self)
+            tmpInt = 0
+            while listIterator.current():
+                item = listIterator.current()
+            
+                if item.isSelected():
+                    tmpInt += 1
+                    
+                if tmpInt >= 2:
+                    multipleSelected = True
+                    break 
+                
+                listIterator += 1
+            
             menuID = popupMenu.insertItem(QIconSet(QPixmap(aliasIconFile)), self.trUtf8("Follow Aliases"), self.enableAliases)
             popupMenu.setItemChecked(menuID, self.aliasDict[server])
                 
@@ -490,29 +662,42 @@ class BrowserWidget(QListView):
         
         
                 # Fill export menu
-                exportMenu.insertItem(self.trUtf8("Item"), self.exportItem)
-                exportMenu.insertItem(self.trUtf8("Subtree"), self.exportItemSubtree)
-                exportMenu.insertItem(self.trUtf8("Subtree with Parents"), self.exportItemAll)
+                if multipleSelected:
+                    exportMenu.insertItem(self.trUtf8("Items"), self.exportItem)
+                    exportMenu.insertItem(self.trUtf8("Subtrees"), self.exportItemSubtree)
+                    exportMenu.insertItem(self.trUtf8("Subtrees with Parents"), self.exportItemAll)
+                else:
+                    exportMenu.insertItem(self.trUtf8("Item"), self.exportItem)
+                    exportMenu.insertItem(self.trUtf8("Subtree"), self.exportItemSubtree)
+                    exportMenu.insertItem(self.trUtf8("Subtree with Parents"), self.exportItemAll)
 
         
                 # Fill delete menu
-                deleteMenu.insertItem(self.trUtf8("Item"), self.deleteItem)
-                deleteMenu.insertItem(self.trUtf8("Subtree"), self.deleteItemsRecursive)
-                deleteMenu.insertItem(self.trUtf8("Subtree without Node"), self.deleteSubtree)
+                if multipleSelected:
+                    deleteMenu.insertItem(self.trUtf8("Items"), self.deleteItem)
+                    deleteMenu.insertItem(self.trUtf8("Subtrees"), self.deleteItemsRecursive)
+                    deleteMenu.insertItem(self.trUtf8("Subtrees without Node"), self.deleteSubtree)
+                else:
+                    deleteMenu.insertItem(self.trUtf8("Item"), self.deleteItem)
+                    deleteMenu.insertItem(self.trUtf8("Subtree"), self.deleteItemsRecursive)
+                    deleteMenu.insertItem(self.trUtf8("Subtree without Node"), self.deleteSubtree)
                 
                 # Fill add menu
                 self.addItemMenu.clear()
+                self.addItemMenu.insertItem(self.trUtf8("Attribute"), self.addAttribute)
+                self.addItemMenu.insertSeparator()
                 templates = TemplateList()
                 for x in templates.templateList:
                     self.addItemMenu.insertItem(x.name, self.addItem)
                     
                 popupMenu.insertSeparator()
-                popupMenu.insertItem(QIconSet(QPixmap(addIconFile)), self.trUtf8("Add Item"), self.addItemMenu)
+                popupMenu.insertItem(QIconSet(QPixmap(addIconFile)), self.trUtf8("Add"), self.addItemMenu)
                 popupMenu.insertSeparator()
                 popupMenu.insertItem(QIconSet(QPixmap(exportIconFile)), self.trUtf8("Export"), exportMenu)
                 popupMenu.insertSeparator()
                 popupMenu.insertItem(QIconSet(QPixmap(delIconFile)), self.trUtf8("Delete"), deleteMenu)
-                
+             
+            self.itemClicked(1, tmpItem)
             popupMenu.exec_loop(point)
 
 ###############################################################################
@@ -542,7 +727,7 @@ class BrowserWidget(QListView):
         parentList = []
         while (item.parent()):
             item = item.parent()
-            parentList.append(self.getFullPath(item))
+            parentList.append(item)
         parentList.reverse()
         del parentList[0]
         return parentList
@@ -562,7 +747,20 @@ class BrowserWidget(QListView):
         template = templates.getTemplate(templateName)
         
         # get server name and basedn from where to add
-        fqn = self.getFullPath(self.selectedItem())
+        selectedItems = []
+        listIterator = QListViewItemIterator(self)
+        while listIterator.current():
+            item = listIterator.current()
+            
+            if item.isSelected():
+                selectedItems.append(item)
+                
+            listIterator += 1
+            
+        if len(selectedItems) == 0:
+            return
+        
+        fqn = self.getFullPath(selectedItems[0])
         tmpList = fqn.split(",")
                 
         serverName = tmpList[-1]
@@ -605,38 +803,54 @@ class BrowserWidget(QListView):
             do not delete the selected item
         """
         
-        currentItem = self.selectedItem()
-        
-        parent = currentItem.parent()
-        fullPath = self.getFullPath(currentItem)
-        success, childrenList, exceptionObject = self.getLdapItemChildren(fullPath, 1)
-        
-        if success:
-            if len(childrenList) > 0:
-                if (not withParent):
-                    del childrenList[0]
+        selectedItems = []
+        listIterator = QListViewItemIterator(self)
+        while listIterator.current():
+            item = listIterator.current()
+            
+            if item.isSelected():
+                selectedItems.append(item)
                 
-                childrenList.sort()
-                #print map(lambda x: x.getDN(), childrenList)
-                deleteDialog = DeleteDialog()
-                deleteDialog.initData(childrenList)
-                deleteDialog.exec_loop()
-                parent.setOpen(0)
-                parent.setOpen(1)
-                self.setSelected(parent, True)
-                self.itemClicked(parent)
+            listIterator += 1
+            
+        if len(selectedItems) == 0:
+            return
+            
+        entryList = []
+        partialResults = False
+        for x in selectedItems:
+            fullPath = self.getFullPath(x)
+            
+            # check if we selected a server name
+            serverName, ldapObject = self.splitPath(fullPath)
+            if len(ldapObject) == 0:
+                continue
+                
+            success, resultList, exceptionObject = self.getLdapItemChildren(fullPath, withParent)
+            
+            if success:
+                entryList.extend(resultList)
             else:
-                dialog = LumaErrorDialog()
-                errorMsg = self.trUtf8("Could not retrieve entry for deletion.<br><br>Reason: ")
+                errorMsg = self.trUtf8("Could not retrieve entry with DN %1 on server %2 for deletion.<br><br>Reason: ")
+                errorMsg = errorMsg.text().arg(ldapObject).arg(serverName)
                 errorMsg.append(str(exceptionObject))
-                dialog.setErrorMessage(errorMsg)
-                dialog.exec_loop()
+                environment.logMessage(LogObject("Error", errorMsg))
+                partialResults = True
             
+        if partialResults:
+            dialog = LumaErrorDialog()
+            errorMsg = self.trUtf8("Could not retrieve all entries for deletion. More information in the logger.")
+            dialog.setErrorMessage(errorMsg)
+            dialog.exec_loop()
+            return
             
+        if len(entryList) == 0:
+            return
             
-            #serverName, selectedObject = self.splitPath(fullPath)
-            #if len(selectedObject) == 0:
-            #    return
+        entryList.sort()
+        deleteDialog = DeleteDialog()
+        deleteDialog.initData(entryList)
+        deleteDialog.exec_loop()
             
 ###############################################################################
 
@@ -646,6 +860,74 @@ class BrowserWidget(QListView):
         
         self.deleteItemsRecursive(0)
 
+###############################################################################
+
+    def keyPressEvent(self, keyEvent):
+        self.setEnabled(False)
+        keyText = unicode(keyEvent.text()).lower()
+        if len(keyText) == 0:
+            keyEvent.ignore()
+            self.setEnabled(True)
+            return
+        
+        # find the first item below the currently selected item
+        selectedItem = self.selectedItem()
+        currentItem = None
+        
+        # Either no item is selected of the can select multiple items.
+        if selectedItem == None:
+            listIterator = QListViewItemIterator(self)
+            while listIterator.current():
+                item = listIterator.current()
+            
+                if item.isSelected():
+                    currentItem = item.itemBelow()
+                    break
+                
+                listIterator += 1
+                
+        # We have one selected item
+        else:
+            currentItem = selectedItem.itemBelow()
+        
+        # Small error prevention
+        if currentItem == None:
+            currentItem = self.firstChild()
+            if currentItem == None:
+                self.setEnabled(True)
+                return
+        
+        # Now we try to find the next item matching our search key.
+        listIterator = QListViewItemIterator(currentItem)
+        item = None
+        while listIterator.current():
+            item = listIterator.current()
+            
+            nodeString = unicode(item.text(0))
+            tmpList = nodeString.split("=")
+            if len(tmpList) < 2:
+                listIterator +=1
+                continue
+                    
+            tmpString = tmpList[1]
+            if len(tmpString) == 0:
+                listIterator +=1
+                continue
+                    
+            if keyTex == tmpString[0].lower():
+                self.ensureItemVisible(item)
+                self.setSelected(item, True)
+                self.itemClicked(item)
+                break
+                
+            listIterator += 1
+            
+        # When re-enabled, we need to get the focus again. Otherwise further 
+        # key presses will be ignored.
+        self.setEnabled(True)
+        self.setFocus()
+        
+        
 ###############################################################################
 
     def deleteLdapEntry(self, serverName, ldapObject):
@@ -701,7 +983,7 @@ class BrowserWidget(QListView):
             item = listIterator.current()
             itemDN = self.getFullPath(item)
             
-            if dnString == itemDN:
+            if itemDN == dnString: 
                 item.setOpen(0)
                 item.setOpen(1)
                 break
@@ -710,7 +992,14 @@ class BrowserWidget(QListView):
         
 ###############################################################################
 
-
+    def addAttribute(self):
+        """ Emits a signal that an attribute is to be added to the current
+        entry.
+        """
+        
+        self.emit(PYSIGNAL("ADD_ATTRIBUTE"), ())
+        
+###############################################################################
 
 class ChildWindow(QMainWindow):
     
