@@ -21,6 +21,7 @@ from base.backend.ServerObject import ServerObject
 from base.backend.ServerList import ServerList
 from base.backend.LumaConnection import LumaConnection
 from base.utils.gui.LumaErrorDialog import LumaErrorDialog
+from base.utils.backend.LogObject import LogObject
 
 class LumaEntryBrowser (LumaEntryBrowserDesign):
 
@@ -80,7 +81,7 @@ class LumaEntryBrowser (LumaEntryBrowserDesign):
         if not (self.serverList == None):
             tmpDict = {}
             for x in self.serverList:
-                if x.tls == 1:
+                if not (x.encryptionMethod == u"None"):
                     tmpDict[x.name] = True
                 else:
                     tmpDict[x.name] = False
@@ -116,7 +117,7 @@ class LumaEntryBrowser (LumaEntryBrowserDesign):
         
         self.setEnabled(True)
         
-        if not (None == self.SERVERMETA):
+        if not (self.SERVERMETA == None):
             self.initBaseBox()
         
             self.emit(PYSIGNAL("about_to_change"), ())
@@ -145,7 +146,7 @@ class LumaEntryBrowser (LumaEntryBrowserDesign):
           
         self.setEnabled(True)
         
-        if None == baseList:
+        if baseList == None:
             self.goButton.setEnabled(False)
             return
         else:
@@ -161,11 +162,14 @@ class LumaEntryBrowser (LumaEntryBrowserDesign):
 ###############################################################################
 
     def search(self, filter=None):
-        self.setEnabled(False)
+        if self.lumaConnection == None:
+            return
         
         if self.lumaConnection.serverMeta == None:
             return
             
+        self.setEnabled(False)
+            
         if not(unicode(self.searchEdit.text()) == ''):
             filter = unicode(self.searchEdit.text())
           
@@ -193,6 +197,7 @@ class LumaEntryBrowser (LumaEntryBrowserDesign):
             errorMsg.append(str(exceptionObject))
             dialog.setErrorMessage(errorMsg)
             dialog.exec_loop()
+            self.setEnabled(True)
             return 
                 
         success, resultList, exceptionObject = self.lumaConnection.search(self.SERVERMETA.currentBase, ldap.SCOPE_SUBTREE, tmpFilter.encode('utf-8'), [self.primaryKey, 'sn', 'givenName'], 0)
@@ -213,6 +218,9 @@ class LumaEntryBrowser (LumaEntryBrowserDesign):
 ###############################################################################
 
     def processResults(self, results):
+        statusBar = qApp.mainWidget().statusBar()
+        tmpMessage = self.trUtf8("Received %1 entries").arg(unicode(len(results)))
+        statusBar.message(tmpMessage, 10000)
         self.data={}
         if not(results == None):
             for x in results:
@@ -242,7 +250,7 @@ class LumaEntryBrowser (LumaEntryBrowserDesign):
                     if tmpData.hasAttribute('sn'):
                         name = tmpData.getAttributeValue('sn', 0)
                     if tmpData.hasAttribute('givenName'):
-                        if not('' == name):
+                        if not(name == ''):
                             name = name + ' '
                         name += tmpData.getAttributeValue('givenName', 0)
                 
@@ -402,9 +410,10 @@ class LumaEntryBrowser (LumaEntryBrowserDesign):
         try:
             config.readfp(open(configFile, 'r'))
         except IOError, e:
-            print "Could not read configuration file. Reason: "
-            print e
-        
+            tmpString =  "Could not read configuration file. Reason:\n"
+            tmpString += str(e)
+            environment.logMessage(LogObject("Debug", tmpString))
+            
         self.filterElements = ["cn", "sn", "givenName", "mail"]
         
         if config.has_option(pluginName, 'filter'):
